/*
 * EDSHOT Change Log
 *
 * rev1
 * -Translations
 * -Switched to SourceMod voting API
 *
 * rev2
 * -Include translation support in matchmodes.txt
 *  This also means that you must update your matchmodes.txt to the new structure!
 *
 *
 * EDSHOT Bug List
 * -It may be possible to do the 'map transition pass' bug here as well. Haven't tried though.
 * -There are times when a tied vote (1-1) passes anyways. I don't see any problems here so I am just going to
 *  assume its another SourceMod voting API bug.
 */

#include <sourcemod>
#include <sdktools>
#include "edlib.sp"

#define MATCHMODES_PATH		"configs/matchmodes.txt"
#define MATCHMODES_PREFIX	"[matchvote] "

#define VOTE_SOUND_START	"ui/beep_synthtone01.wav"
#define VOTE_SOUND_PASS		"ui/menu_enter05.wav"
#define VOTE_SOUND_FAIL		"ui/beep_error01.wav"
#define VOTE_TIMEOUT		20

new Handle:g_hModesKV 		= INVALID_HANDLE;
new Handle:g_hCvarCfgWaitTime 	= INVALID_HANDLE;
new String:g_sCfg[64];
new String:tBuffer[128];
new String:sLanguage[6];
bool IsMatchVoteInProgress	= false;

public Plugin:myinfo = 
{
	name = "Match Vote",
	author = "vintik, epilimic, EDSHOT",
	description = "type !match to vote a new mode",
	version = "1.3-rev2"
}

public void OnPluginStart()
{
	new String:sBuffer[128];

	GetGameFolderName(sBuffer, sizeof(sBuffer));
	if (!StrEqual(sBuffer, "left4dead", false) && !StrEqual(sBuffer, "left4dead2", false))
	{
		SetFailState("This plugin is only supported by the Left 4 Dead series.");
	}

	g_hModesKV = CreateKeyValues("MatchModes");
	BuildPath(Path_SM, sBuffer, sizeof(sBuffer), MATCHMODES_PATH);
	if (!FileToKeyValues(g_hModesKV, sBuffer))
	{
		SetFailState("Couldn't load configs/matchmodes.txt!");
	}

	RegConsoleCmd("sm_match", MatchRequest);
	RegConsoleCmd("sm_matchvote", VoteRedraw);
	g_hCvarCfgWaitTime = CreateConVar("sm_match_cfgwaittime", "3", "Number of seconds before executing match configuration", FCVAR_PLUGIN, true, 1.0, false, 1.0);

	PrecacheSound(VOTE_SOUND_START, true);
	PrecacheSound(VOTE_SOUND_PASS, true);
	PrecacheSound(VOTE_SOUND_FAIL, true);

	LoadTranslations("common.phrases");
	LoadTranslations("matchvote.phrases");
}

public void OnConfigsExecuted()
{
	PrecacheSound(VOTE_SOUND_START, true);
	PrecacheSound(VOTE_SOUND_PASS, true);
	PrecacheSound(VOTE_SOUND_FAIL, true);

	// 
	// In case someone called the vote right before the map transition.
	// - EDSHOT
	// 
	IsMatchVoteInProgress = false;
}

public Action:MatchRequest(client, args)
{
	if (client != 0 && IsClientInGame(client)) {}
	else { return Plugin_Handled; }

	if (args > 0)
	{
		//config specified
		new String:sCfg[64];
		new String:sBuffer[128];
		GetCmdArg(1, sCfg, sizeof(sCfg));

		// exec
		//BuildPath(Path_SM, sBuffer, sizeof(sBuffer), "../../cfg/%s.cfg", sCfg);

		// sm_execcfg
		BuildPath(Path_SM, sBuffer, sizeof(sBuffer), "%s.cfg", sCfg);

		if (FileExists(sBuffer))
		{
			FindConfigName(sCfg, sBuffer, sizeof(sBuffer), client);
			if (StartMatchVote(client, sBuffer))
			{
				strcopy(g_sCfg, sizeof(g_sCfg), sCfg);
				PrintToChatAllTranslated1I("PlayerStartedVote", MATCHMODES_PREFIX, "", client);
				PrintToChatAll(sBuffer);
				//PrintToChatAllTranslated("MatchVoteCommandHint", MATCHMODES_PREFIX, ".");
			}
			return Plugin_Handled;
		}
		else
		{
			Format(tBuffer, sizeof(tBuffer), "%s%T!", MATCHMODES_PREFIX, "FileDoesNotExist", client, sBuffer);
			PrintToChat(client, tBuffer);
			return Plugin_Handled;
		}
	}
	else
	{
		//show main menu
		MatchModeMenu(client);
	}

	return Plugin_Handled;
}

public bool FindConfigName(const String:cfg[], String:name[], int maxlength, int client)
{
	KvRewind(g_hModesKV);
	if (KvGotoFirstSubKey(g_hModesKV))
	{
		do
		{
			if (KvJumpToKey(g_hModesKV, cfg))
			{
				GetClientLanguageCode(client);
				KvGetString(g_hModesKV, sLanguage, name, maxlength);
				if (StrEqual(name, "", false)) KvGetString(g_hModesKV, "en", name, maxlength);
				return true;
			}
		} while (KvGotoNextKey(g_hModesKV));
	}
	return false;
}

public Action:MatchModeMenu(client)
{
	new Handle:hMenu = CreateMenu(MatchModeMenuHandler);
	Format(tBuffer, sizeof(tBuffer), "%T:", "MatchModeMenuTitle", client);
	SetMenuTitle(hMenu, tBuffer);
	new String:sBuffer[128];
	new String:sInfo[128];
	GetClientLanguageCode(client);
	KvRewind(g_hModesKV);
	if (KvGotoFirstSubKey(g_hModesKV))
	{
		do
		{
			KvSavePosition(g_hModesKV);
			KvGotoFirstSubKey(g_hModesKV);
			KvGetString(g_hModesKV, sLanguage, sInfo, sizeof(sInfo));
			if (StrEqual(sInfo, "", false)) KvGetString(g_hModesKV, "en", sInfo, sizeof(sInfo));
			KvGoBack(g_hModesKV);
			KvGetSectionName(g_hModesKV, sBuffer, sizeof(sBuffer));
			AddMenuItem(hMenu, sBuffer, sInfo);
		} while (KvGotoNextKey(g_hModesKV));
	}
	DisplayMenu(hMenu, client, 20);
}

public MatchModeMenuHandler(Handle:menu, MenuAction:action, param1, param2)
{
	if (action == MenuAction_Select)
	{
		new String:sInfo[128];
		new String:sBuffer[128];
		GetClientLanguageCode(param1);
		GetMenuItem(menu, param2, sInfo, sizeof(sInfo));
		KvRewind(g_hModesKV);
		if (KvJumpToKey(g_hModesKV, sInfo) && KvGotoFirstSubKey(g_hModesKV))
		{
			new Handle:hMenu = CreateMenu(ConfigsMenuHandler);

			// sInfo is the fake section name.
			// sBuffer (after this comment) will become the translated section name (with $NAME key)
			KvGetString(g_hModesKV, sLanguage, sBuffer, sizeof(sBuffer));
			if (StrEqual(sBuffer, "", false)) KvGetString(g_hModesKV, "en", sBuffer, sizeof(sBuffer));
			KvGotoNextKey(g_hModesKV);

			Format(tBuffer, sizeof(tBuffer), "%T:", "MatchModeMenuSelectConfigTitle", param1, sBuffer);
			SetMenuTitle(hMenu, tBuffer);
			do
			{
				KvGetSectionName(g_hModesKV, sInfo, sizeof(sInfo));
				KvGetString(g_hModesKV, sLanguage, sBuffer, sizeof(sBuffer));
				if (StrEqual(sBuffer, "", false)) KvGetString(g_hModesKV, "en", sBuffer, sizeof(sBuffer));
				AddMenuItem(hMenu, sInfo, sBuffer);
			} while (KvGotoNextKey(g_hModesKV));
			DisplayMenu(hMenu, param1, 20);
		}
		else
		{
			Format(tBuffer, sizeof(tBuffer), "%T.", "NoConfigsFound", param1);
			PrintToChat(param1, tBuffer);
			MatchModeMenu(param1);
		}
	}
	if (action == MenuAction_End)
	{
		CloseHandle(menu);
	}
}

public ConfigsMenuHandler(Handle:menu, MenuAction:action, param1, param2)
{
	if (action == MenuAction_Select)
	{
		new String:sInfo[128];
		new String:sBuffer[128];
		GetMenuItem(menu, param2, sInfo, sizeof(sInfo), _, sBuffer, sizeof(sBuffer));
		if (StartMatchVote(param1, sBuffer))
		{
			strcopy(g_sCfg, sizeof(g_sCfg), sInfo);
			PrintToChatAllTranslated1I("PlayerStartedVote", MATCHMODES_PREFIX, "", param1);
			PrintToChatAll(sBuffer);
			//PrintToChatAllTranslated("MatchVoteCommandHint", MATCHMODES_PREFIX, ".");
		}
		else
		{
			MatchModeMenu(param1);
		}
	}
	if (action == MenuAction_End)
	{
		CloseHandle(menu);
	}
	if (action == MenuAction_Cancel)
	{
		MatchModeMenu(param1);
	}
}

public bool StartMatchVote(client, const String:cfgname[])
{
	if (GetClientTeam(client) == 1)
	{
		Format(tBuffer, sizeof(tBuffer), "%s%T.", MATCHMODES_PREFIX, "SpectatorNotAllowedToVote", client);
		PrintToChat(client, tBuffer);
		return false;
	}
	// 
	// IsVoteInProgress() doesn't work.
	// 
	// IsVoteInProgress(Handle:menu) can't be done since the menu is created within this block.
	// Even then, the documentation says "Deprecated; no longer used."
	// - EDSHOT
	// 
	if (!IsMatchVoteInProgress)
	{
		new iNumPlayers;
		decl iPlayers[MaxClients];

		//list of non-spectators players
		for (int i = 1; i <= MaxClients; i++)
		{
			if (!IsClientInGame(i) || IsFakeClient(i) || (GetClientTeam(i) == 1))
			{
				continue;
			}
			iPlayers[iNumPlayers++] = i;
		}

		Menu menu = new Menu(VoteHandler);

		Format(tBuffer, sizeof(tBuffer), "%T", "ChangeTheModeQuestion", client, cfgname);
		menu.SetTitle(tBuffer);
		Format(tBuffer, sizeof(tBuffer), "%T", "Yes", client);
		menu.AddItem("", tBuffer);
		Format(tBuffer, sizeof(tBuffer), "%T", "No", client);
		menu.AddItem("", tBuffer);
		menu.ExitButton = false;

		EmitSoundToAll(VOTE_SOUND_START, -2, 0, 75, 0, 1.0, 100, -1, NULL_VECTOR, NULL_VECTOR, true, 0.0);
		menu.DisplayVote(iPlayers, MaxClients, VOTE_TIMEOUT);

		IsMatchVoteInProgress = true;
		return true;
	}
	Format(tBuffer, sizeof(tBuffer), "%s%T.", MATCHMODES_PREFIX, "MatchVoteCannotStart", client);
	PrintToChat(client, tBuffer);
	return false;
}

public Action:VoteRedraw(client, args)
{
	if (IsClientInVotePool(client))
	{
		RedrawClientVoteMenu(client, false);
	}
	else
	{
		Format(tBuffer, sizeof(tBuffer), "%s%T.", MATCHMODES_PREFIX, "MatchVoteCommandAlreadyVoted", client);
		PrintToChat(client, tBuffer);
	}

	return Plugin_Handled;
}

public int VoteHandler(Menu menu, MenuAction action, int param1, int param2)
{
	switch (action)
	{
		case MenuAction_End:
		{
			IsMatchVoteInProgress = false;
		}
		case MenuAction_VoteEnd:
		{
			if (param1 == 0)
			{
				EmitSoundToAll(VOTE_SOUND_PASS, -2, 0, 75, 0, 1.0, 100, -1, NULL_VECTOR, NULL_VECTOR, true, 0.0);
				PrintToChatAllTranslated1I("ChangeModeAfterSeconds", MATCHMODES_PREFIX, ".", GetConVarInt(g_hCvarCfgWaitTime));
				IsMatchVoteInProgress = false;
				CreateTimer((GetConVarInt(g_hCvarCfgWaitTime) * 1.0), COOL_DOWN);
			}
			else
			{
				EmitSoundToAll(VOTE_SOUND_FAIL, -2, 0, 75, 0, 1.0, 100, -1, NULL_VECTOR, NULL_VECTOR, true, 0.0);
				PrintToChatAllTranslated("VoteFailed", MATCHMODES_PREFIX, ".");
				IsMatchVoteInProgress = false;
			}
		}
	}
}

public Action:COOL_DOWN(Handle:timer)
{
	// exec (doesn't show anything unless convar has FCVAR_NOTIFY)
	//ServerCommand("exec %s", g_sCfg);

	// sm_execcfg (shows executed config)
	ServerCommand("sm_execcfg %s.cfg", g_sCfg);
}

// 
// Do not replace these numbers!
// The numbers in languages.cfg are wrong and must be obtained with GetLanguageByCode()
// I made a plugin for this and will use the correct numbers here.
// - EDSHOT
// 
public void GetClientLanguageCode(int client)
{
	switch (GetClientLanguage(client))
	{
		// English
		case 0: sLanguage = "en";

		// Arabic
		case 1: sLanguage = "ar";

		// Brazilian Portuguese
		case 2: sLanguage = "pt";

		// Bulgarian
		case 3: sLanguage = "bg";

		// Czech
		case 4: sLanguage = "cze";

		// Danish
		case 5: sLanguage = "da";

		// Dutch
		case 6: sLanguage = "nl";

		// Finnish
		case 7: sLanguage = "fi";

		// French
		case 8: sLanguage = "fr";

		// German
		case 9: sLanguage = "de";

		// Greek
		case 10: sLanguage = "el";

		// Hebrew
		case 11: sLanguage = "he";

		// Hungarian
		case 12: sLanguage = "hu";

		// Italian
		case 13: sLanguage = "it";

		// Japanese
		case 14: sLanguage = "jp";

		// Korean
		case 15: sLanguage = "ko";

		// Latvian
		case 16: sLanguage = "lv";

		// Lithuanian
		case 17: sLanguage = "lt";

		// Norwegian
		case 18: sLanguage = "no";

		// Polish
		case 19: sLanguage = "pl";

		// Portuguese
		case 20: sLanguage = "pt_p";

		// Romanian
		case 21: sLanguage = "ro";

		// Russian
		case 22: sLanguage = "ru";

		// Chinese (Simplified)
		case 23: sLanguage = "chi";

		// Slovak
		case 24: sLanguage = "sk";

		// Spanish
		case 25: sLanguage = "es";

		// Swedish
		case 26: sLanguage = "sv";

		// Chinese (Traditional)
		case 27: sLanguage = "zho";

		// Thai
		case 28: sLanguage = "th";

		// Turkish
		case 29: sLanguage = "tr";

		// Ukrainian
		case 30: sLanguage = "ua";

		// Vietnamese
		case 31: sLanguage = "vi";

		// English
		default: sLanguage = "en";
	}
}
